/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('ExpandingRow', function () {

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('expanding-row', function () {
      done();
    });
  });

  it('can render', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.ExpandingRow,
      {
        classes: { extraClass: true } },
      React.createElement(
        'span',
        null,
        'closed'
      ),
      React.createElement(
        'span',
        null,
        'open'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var expected = React.createElement(
      'li',
      { className: 'expanding-row twelve-col extraClass',
        onClick: instance._toggle },
      React.createElement(
        'div',
        { className: 'expanding-row__initial twelve-col no-margin-bottom' },
        React.createElement(
          'span',
          null,
          'closed'
        )
      ),
      React.createElement(
        'div',
        { className: 'expanding-row__expanded twelve-col',
          style: { height: '0px', opacity: 0 } },
        React.createElement(
          'div',
          { className: 'twelve-col no-margin-bottom',
            ref: 'inner' },
          React.createElement(
            'span',
            null,
            'open'
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can toggle to the expanded view', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.ExpandingRow,
      null,
      React.createElement(
        'span',
        null,
        'closed'
      ),
      React.createElement(
        'span',
        null,
        'open'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    // Mock the ref.
    instance.refs = { inner: { offsetHeight: 10 } };
    output.props.onClick();
    output = renderer.getRenderOutput();
    var expected = React.createElement(
      'li',
      { className: 'expanding-row twelve-col expanding-row--expanded',
        onClick: instance._toggle },
      output.props.children
    );
    assert.deepEqual(output, expected);
  });

  it('can initially be expanded', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.ExpandingRow,
      {
        expanded: true },
      React.createElement(
        'span',
        null,
        'closed'
      ),
      React.createElement(
        'span',
        null,
        'open'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    // Mock the ref.
    instance.refs = { inner: { offsetHeight: 10 } };
    // The shallow renderer does not call componentDidMount, so call it
    // manually.
    instance.componentDidMount();
    var output = renderer.getRenderOutput();
    var expected = React.createElement(
      'li',
      { className: 'expanding-row twelve-col expanding-row--expanded',
        onClick: instance._toggle },
      output.props.children
    );
    assert.deepEqual(output, expected);
  });
});